@@ -3,7 +3,7 @@ use warnings;
 
 package DBIx::Class::ResultSet::RecursiveUpdate;
 {
-  $DBIx::Class::ResultSet::RecursiveUpdate::VERSION = '0.31';
+  $DBIx::Class::ResultSet::RecursiveUpdate::VERSION = '0.33';
 }
 
 # ABSTRACT: like update_or_create - but recursive
@@ -40,8 +40,9 @@ sub recursive_update {
 
 package DBIx::Class::ResultSet::RecursiveUpdate::Functions;
 {
-  $DBIx::Class::ResultSet::RecursiveUpdate::Functions::VERSION = '0.31';
+  $DBIx::Class::ResultSet::RecursiveUpdate::Functions::VERSION = '0.33';
 }
+
 use Carp::Clan qw/^DBIx::Class|^HTML::FormHandler|^Try::Tiny/;
 use Scalar::Util qw( blessed );
 use List::MoreUtils qw/ any all/;
@@ -82,9 +83,6 @@ sub recursive_update {
         my @pks = map { $updates->{$_} } @pks;
         $object = $self->find( @pks, { key => 'primary' } );
     }
-    elsif ( !defined $object && exists $updates->{id} ) {
-        $object = $self->find( $updates->{id}, { key => 'primary' } );
-    }
 
     my %fixed_fields = map { $_ => 1 } @$fixed_fields
         if $fixed_fields;
@@ -109,6 +107,14 @@ sub recursive_update {
         $object = $self->find( $updates, { key => 'primary' } );
     }
 
+    # try to construct a new row object with all given update attributes
+    # and use it to find the row in the database
+    if ( !defined $object ) {
+        try {
+            $object = $self->new_result($updates)->get_from_storage;
+        };
+    }
+
     $object = $self->new_result( {} )
         unless defined $object;
 
@@ -529,6 +535,12 @@ sub _master_relation_cond {
             return _inner( $source, $new_cond, @foreign_ids );
         }
     }
+
+    # we have a custom join condition, so update afterward
+    elsif ( ref $cond eq 'CODE' ) {
+        return 0;
+    }
+
     else {
         $source->throw_exception( "unhandled relation condition " . ref($cond) );
     }
@@ -549,7 +561,7 @@ DBIx::Class::ResultSet::RecursiveUpdate - like update_or_create - but recursive
 
 =head1 VERSION
 
-version 0.31
+version 0.33
 
 =head1 SYNOPSIS
 
